/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * Interface for warning about deprecated operations.  Consumers should
 * attach this interface to the channel's notification callbacks/loadgroup.
 */
[uuid(665c5124-2c52-41ba-ae72-2393f8e76c25)]
interface nsIDeprecationWarner : nsISupports
{
    /**
     * Issue a deprecation warning.
     *
     * @param aWarning a warning code as declared in nsDeprecatedOperationList.h.
     * @param aAsError optional boolean flag indicating whether the warning
     *                 should be treated as an error.
     */
    void issueWarning(in uint32_t aWarning, [optional] in bool aAsError);
};
