#define MOZ_SOURCE_STAMP 61519976b35f2947eeaabefcad83186b7e004167
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/61519976b35f2947eeaabefcad83186b7e004167
